package org.eclipse.bpel.model.util.tests;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;

import junit.framework.TestCase;

import org.eclipse.bpel.model.PartnerLink;
import org.eclipse.bpel.model.Process;
import org.eclipse.bpel.model.partnerlinktype.PartnerLinkType;
import org.eclipse.bpel.model.proxy.PartnerLinkTypeProxy;
import org.eclipse.bpel.model.tests.TestProject;
import org.eclipse.bpel.runtimes.IBPELModuleFacetConstants;
import org.eclipse.bpel.runtimes.module.BPELModuleArtifact;
import org.eclipse.bpel.ui.util.BPELReader;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.wst.server.core.IModule;
import org.eclipse.wst.server.core.IModuleArtifact;
import org.eclipse.wst.server.core.internal.Module;

public class WSDLUtilTest extends TestCase {
	
	private final static String HOST = "localhost";
	private final static int HTTP_PORT = 8080;
	
	private static final String PLUGIN_ID = "org.eclipse.bpel.model.tests";
	private final static String ARTS_BPEL_FILE_PATH = "test_files/ArtifactsServiceProcess.bpel";
	private final static String ARTS_EXT_FILE_PATH = "test_files/ArtifactsServiceProcess.bpelex";
	private final static String ARTS_WSDL_FILE_PATH = "test_files/ArtifactsServiceProcess.wsdl";
	private final static String ARTS_ART_FILE_PATH = "test_files/ArtifactsServiceProcessArtifacts.wsdl";
	
	private String m_ArtsBPELFileLocation;
	private String m_ArtsExtFileLocation;
	private String m_ArtsWSDLFileLocation;
	private String m_ArtsArtifactsFileLocation;
	private TestProject testProject = null;

	public WSDLUtilTest(String name) {
		super(name);
	}

	protected void setUp() throws Exception {
		m_ArtsBPELFileLocation = getPluginAbsolutePath(PLUGIN_ID).concat(ARTS_BPEL_FILE_PATH);
		m_ArtsExtFileLocation = getPluginAbsolutePath(PLUGIN_ID).concat(ARTS_EXT_FILE_PATH);
		m_ArtsWSDLFileLocation = getPluginAbsolutePath(PLUGIN_ID).concat(ARTS_WSDL_FILE_PATH);
		m_ArtsArtifactsFileLocation = getPluginAbsolutePath(PLUGIN_ID).concat(ARTS_BPEL_FILE_PATH);
		
		if (testProject != null) testProject.dispose();
		testProject = new TestProject();
	}

	protected void tearDown() throws Exception {
		m_ArtsBPELFileLocation = null;
		m_ArtsExtFileLocation = null;
		m_ArtsWSDLFileLocation = null;
		testProject.dispose();
		testProject = null;
	}
	
	/**
	 * Exposing the bug. 
	 * 
	 * 
	 * TODO remove all references to ABR plug-in code by inlining
	 */
	public void testResolveProxyPLT() throws CoreException, IOException {
		IFile bpelFile = createArtifactsServiceTestProject(); 
		BPELModuleArtifact bpelModuleArtifact = createBPELModuleArtifact(bpelFile);
		assertNotNull(bpelModuleArtifact);
		
		Process bpelProcess = getBpelProcess(bpelModuleArtifact);
		EList bpelPartnerLinks = bpelProcess.getPartnerLinks().getChildren();
		assertTrue(2 == bpelPartnerLinks.size());
		
		/* somePartner bpel partnerLink */
		PartnerLink bpelPartnerLink = (PartnerLink) bpelPartnerLinks.get(1);
		assertNotNull(bpelPartnerLink);
		
		PartnerLinkType plt = bpelPartnerLink.getPartnerLinkType(); // this should
																	// resolve the proxy
		assertNotNull(plt);
		assertTrue(plt instanceof PartnerLinkTypeProxy); // it's still a proxy
		assertNotNull(plt.getName()); // can't get its name, it's still a proxy
		
		// what we want
//		String id = bpelPartnerLink.getPartnerLinkType().getID();
//		assertNotNull(id);
	}
	
	private Process getBpelProcess(IModuleArtifact moduleArtifact) {
		IFile bpelFile = ((BPELModuleArtifact) moduleArtifact).getFile();
		if (bpelFile == null) return null;
		
		ResourceSet resourceSet = new ResourceSetImpl();
		URI uri = URI.createPlatformResourceURI(bpelFile.getFullPath().toString());
		Resource bpelResource = resourceSet.getResource(uri, true);
		BPELReader reader = new BPELReader();
		reader.read(bpelResource, bpelFile, resourceSet);
		
		return reader.getProcess();	
	}
	
	
	private String getPluginAbsolutePath(String pluginID) {
		String location = Platform.getBundle(pluginID).getLocation();
		return location.split("@")[1];
	}
	
	private IFile createArtifactsServiceTestProject() throws IOException, CoreException {
    	IFile mockFile1 = testProject.getProject().getFile(new Path("ArtifactsServiceProcess.bpel"));
		InputStream in = new FileInputStream(m_ArtsBPELFileLocation);
		mockFile1.create(in, true, null);
		
		IFile mockFile = testProject.getProject().getFile(new Path("ArtifactsServiceProcess.bpelex"));
		in = new FileInputStream(m_ArtsExtFileLocation);
		mockFile.create(in, true, null);

		mockFile = testProject.getProject().getFile(new Path("ArtifactsServiceProcess.wsdl"));
		in = new FileInputStream(m_ArtsWSDLFileLocation);
		mockFile.create(in, true, null);
		
		mockFile = testProject.getProject().getFile(new Path("ArtifactsServiceProcessArtifacts.wsdl"));
		in = new FileInputStream(m_ArtsArtifactsFileLocation);
		mockFile.create(in, true, null);
		
		return mockFile1;
    }
	
	private BPELModuleArtifact createBPELModuleArtifact(IFile bpelFile) {
		IModule module = new Module(
				null,
				testProject.getProject().getName().concat(bpelFile.getName()),
				bpelFile.getName(),
				IBPELModuleFacetConstants.BPEL20_MODULE_TYPE,
				"2.0",
				testProject.getProject());

		return new BPELModuleArtifact(module, bpelFile);
	}

}
